@@ -19,8 +19,6 @@
 
 this source is for now limited to a read-only CWUser source
 
-
-
 Part of the code is coming form Zope's LDAPUserFolder
 
 Copyright (c) 2004 Jens Vagelpohl.
@@ -242,7 +240,7 @@ directory (default to once a day).',
                             if emailaddr == ldapemailaddr:
                                 break
                         else:
-                            self.info('updating email address of user %s to %s',
+                            self.debug('updating email address of user %s to %s',
                                       extid, ldapemailaddr)
                             emailrset = execute('EmailAddress A WHERE A address %(addr)s',
                                                 {'addr': ldapemailaddr})
@@ -278,7 +276,10 @@ directory (default to once a day).',
         to fetch the salt first
         """
         self.info('ldap authenticate %s', login)
-        if password is None:
+        if not password:
+            # On Windows + ADAM this would have succeeded (!!!)
+            # You get Authenticated as: 'NT AUTHORITY\ANONYMOUS LOGON'.
+            # we really really don't want that
             raise AuthenticationError()
         searchfilter = [filter_format('(%s=%s)', (self.user_login_attr, login))]
         searchfilter.extend([filter_format('(%s=%s)', ('objectClass', o))
@@ -294,9 +295,12 @@ directory (default to once a day).',
         # check password by establishing a (unused) connection
         try:
             self._connect(user, password)
-        except Exception, ex:
-            self.info('while trying to authenticate %s: %s', user, ex)
+        except ldap.LDAPError, ex:
             # Something went wrong, most likely bad credentials
+            self.info('while trying to authenticate %s: %s', user, ex)
+            raise AuthenticationError()
+        except Exception:
+            self.error('while trying to authenticate %s', user, exc_info=True)
             raise AuthenticationError()
         return self.extid2eid(user['dn'], 'CWUser', session)
 
@@ -504,7 +508,7 @@ directory (default to once a day).',
     def _search(self, session, base, scope,
                 searchstr='(objectClass=*)', attrs=()):
         """make an ldap query"""
-        self.info('ldap search %s %s %s %s %s', self.uri, base, scope, searchstr, list(attrs))
+        self.debug('ldap search %s %s %s %s %s', self.uri, base, scope, searchstr, list(attrs))
         cnx = session.pool.connection(self.uri).cnx
         try:
             res = cnx.search_s(base, scope, searchstr, attrs)
@@ -520,12 +524,12 @@ directory (default to once a day).',
                 self.repo.delete_info(session, entity, self.uri, base)
                 self.reset_cache()
             return []
-##         except ldap.REFERRAL, e:
-##             cnx = self.handle_referral(e)
-##             try:
-##                 res = cnx.search_s(base, scope, searchstr, attrs)
-##             except ldap.PARTIAL_RESULTS:
-##                 res_type, res = cnx.result(all=0)
+        # except ldap.REFERRAL, e:
+        #     cnx = self.handle_referral(e)
+        #     try:
+        #         res = cnx.search_s(base, scope, searchstr, attrs)
+        #     except ldap.PARTIAL_RESULTS:
+        #         res_type, res = cnx.result(all=0)
         result = []
         for rec_dn, rec_dict in res:
             # When used against Active Directory, "rec_dict" may not be
@@ -553,7 +557,7 @@ directory (default to once a day).',
             self._cache[rec_dn] = rec_dict
             result.append(rec_dict)
         #print '--->', result
-        self.info('ldap built results %s', result)
+        self.debug('ldap built results %s', len(result))
         return result
 
     def before_entity_insertion(self, session, lid, etype, eid):
@@ -564,7 +568,7 @@ directory (default to once a day).',
         This method must return the an Entity instance representation of this
         entity.
         """
-        self.info('ldap before entity insertion')
+        self.debug('ldap before entity insertion')
         entity = super(LDAPUserSource, self).before_entity_insertion(session, lid, etype, eid)
         res = self._search(session, lid, BASE)[0]
         for attr in entity.e_schema.indexable_attributes():
@@ -575,7 +579,7 @@ directory (default to once a day).',
         """called by the repository after an entity stored here has been
         inserted in the system table.
         """
-        self.info('ldap after entity insertion')
+        self.debug('ldap after entity insertion')
         super(LDAPUserSource, self).after_entity_insertion(session, dn, entity)
         for group in self.user_default_groups:
             session.execute('SET X in_group G WHERE X eid %(x)s, G name %(group)s',
