@@ -91,22 +91,35 @@ jQuery.extend(Deferred.prototype, {
 var JSON_BASE_URL = baseuri() + 'json?';
 
 //============= utility function handling remote calls responses. ==============//
-function _loadAjaxHtmlHead(node, head, tag, srcattr) {
+function _loadAjaxHtmlHead($node, $head, tag, srcattr) {
     var loaded = [];
     var jqtagfilter = tag + '[' + srcattr + ']';
     jQuery('head ' + jqtagfilter).each(function(i) {
         loaded.push(this.getAttribute(srcattr));
     });
-    node.find(tag).each(function(i) {
-        if (this.getAttribute(srcattr)) {
-            if (jQuery.inArray(this.getAttribute(srcattr), loaded) == -1) {
-                jQuery(this).appendTo(head);
+    $node.find(tag).each(function(i) {
+	 var url = this.getAttribute(srcattr);
+        if (url) {
+            if (jQuery.inArray(url, loaded) == -1) {
+		if (srcattr == 'src') {
+		    // special case for <script> tags: jQuery append method
+		    // script nodes don't appears in the DOM (See comments on
+		    // http://api.jquery.com/append/), which cause undesired
+		    // duplicated load in our case. Use bare DOM api to avoid
+		    // this.
+		    var s = document.createElement("script");
+		    s.type = "text/javascript";
+		    s.src = url;
+		    $head[0].appendChild(s);
+		} else {
+		    jQuery(this).appendTo($head);
+		}
             }
         } else {
-            jQuery(this).appendTo(head);
+            jQuery(this).appendTo($head);
         }
     });
-    node.find(jqtagfilter).remove();
+    $node.find(jqtagfilter).remove();
 }
 
 /**
@@ -241,7 +254,7 @@ jQuery.fn.loadxhtml = function(url, form, reqtype, mode, cursor) {
         $(node).removeClass("hidden");
         if (mode == 'swap') {
             var origId = node.id;
-            node = swapDOM(node, domnode);
+            node = cw.swapDOM(node, domnode);
             if (!node.id) {
                 node.id = origId;
             }
@@ -255,16 +268,16 @@ jQuery.fn.loadxhtml = function(url, form, reqtype, mode, cursor) {
             callback = callback.apply(this, [domnode]);
         }
     });
+    d.addErrback(remoteCallFailed);
     if (cursor) {
         d.addCallback(resetCursor);
         d.addErrback(resetCursor);
-        d.addErrback(remoteCallFailed);
     }
     return d;
 }
 
 /**
- * .. function:: loadRemote(url, form, reqtype='GET', async=true)
+ * .. function:: loadRemote(url, form, reqtype='GET', sync=false)
  *
  * Asynchronously (unless `async` argument is set to false) load an url or path
  * and return a deferred whose callbacks args are decoded according to the
