@@ -25,6 +25,7 @@ from logilab.mtconverter import xml_escape
 
 from cubicweb.selectors import none_rset, match_user_groups
 from cubicweb.view import StartupView
+from cubicweb.web.views import actions
 
 def dict_to_html(w, dict):
     # XHTML doesn't allow emtpy <ul> nodes
@@ -37,10 +38,17 @@ def dict_to_html(w, dict):
 
 
 
+class SiteInfoAction(actions.ManagersAction):
+    __regid__ = 'siteinfo'
+    __select__ = match_user_groups('users','managers')
+    title = _('info')
+    order = 30
+
+
 class ProcessInformationView(StartupView):
     """display various web server /repository information"""
     __regid__ = 'info'
-    __select__ = none_rset() & match_user_groups('managers')
+    __select__ = none_rset() & match_user_groups('managers', 'users')
 
     title = _('server information')
     cache_max_age = 0
